@@ -1,3 +1,22 @@
+
+# ProctorServ Python API - A Python Wrapper for ProctorCam's Proctoring Services
+# Copyright (C) 2013 ProctorCam Inc.
+#
+# This file is part of ProctorServ Python API.
+#
+# ProctorServ Python API is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as
+# published by the Free Software Foundation, either version 3 of the
+# License, or (at your option) any later version.
+#
+# ProctorServ Python API is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public License
+# along with ProctorServ Python API.  If not, see <http://www.gnu.org/licenses/>.
+
 from hashed_authenticator import HashedAuthenticator
 from exception import MissingRequiredParameterException
 from exception import InvalidClientException
@@ -68,6 +87,106 @@ class ProctorservApi(object):
 
     return self.time_slot_responses(response)
 
+  # get_access_code
+  #   Will return a object containing access code and its expiration date time based
+  #   on a session ID provided in the request
+  #
+  #   required parameters:
+  #   - session_id     Integer
+  #
+  def get_access_code(self, options):
+    self.requires_of(options, ['session_id'])
+    url = "%s://%s/api/access_code/get_access_code" %(self.service_protocol, self.service_url)
+
+    rest = RestRequestClient()
+    response = rest.make_post_request(url, self.api_identifier, self.shared_secret, options) 
+
+    return self.get_access_code_responses(response)
+
+  # reset_access_code
+  #   Will return an object containing new access code for a session
+  #   based on a session ID provided in the request
+  #
+  #   required parameters:
+  #   - session_id     Integer
+  #
+  def reset_access_code(self, options):
+    self.requires_of(options, ['session_id'])
+    url = "%s://%s/api/access_code/reset_access_code" %(self.service_protocol, self.service_url)
+
+    rest = RestRequestClient()
+    response = rest.make_post_request(url, self.api_identifier, self.shared_secret, options) 
+
+    return self.reset_access_code_responses(response)
+
+
+
+
+  ## data_retrievals
+  #
+  #   Will return data associated with a list of sessions. 
+  #   The information returned is dependant on the elements
+  #   provided in the request
+  #
+  #   required parameters:
+  #     - ids (session id's you would like to obtain data from)                        - Array of IDs (string)
+  #     - elements (the type of data that you would like to retrieve for each session) - Array of elements (string)
+  #
+  #   optional elements:     Select one or more elements to receive data, if left blank no data will be returned
+  #     - review_url         (url to proctorcam to review the past session)
+  #     - video_url          (url to download the sessions video)
+  #     - id_photo_url       (url to download the photo of the id provided by the sessions test taker)
+  #     - headshot_photo_url (url to download the photo of test taker)
+  #     - session_activity   (an array of all session events associated with the session)
+  #
+  #    example:
+  #     - params = {"ids" => ["393", "392"], "elements" => ["id_photo_url", "headshot_photo_url", "session_activity", "video_url"]}
+  #
+  def data_retrievals(self,options):
+    self.requires_of(options, ['ids', 'elements'])
+    url = "%s://%s/api/data_retrievals/sessions" %(self.service_protocol, self.service_url)
+
+    rest = RestRequestClient()
+    response = rest.make_post_request(url, self.api_identifier, self.shared_secret, options)
+
+    return self.data_retrievals_response(response)
+
+  ## dataDeletions
+  #
+  #   Will delete all identifying data associated with a set of sessions
+  #   that are scoped to the smallest possible set by any of the following parameters
+  #   optional parameters:
+  #     - ids (only delete from this list) eg [123,124,125]
+  #     - customer_subject_id (only delete sessions associated with a customer's subject id)
+  #     - customer_client_subject_id (only delete sessions associated with a customer's client's subject id)
+  #     - client_code (API identifier for customer client)
+  #     - exam_code
+  #     - lower_bound - UTC Datetime Object (UTC seconds since epoch after which to delete sessions by reservation time. Defaults to 0)
+  #     - upper_bound - UTC Datetime Object (UTC seconds since epoch before which to delete sessions by reservation time.
+  #                                          Defaults to max 32 bit unsigned int)
+  #
+  #   Note that if no parameters are passed, no sessions will be deleted.
+  #
+  #   returns an array of session ids that were successfully deleted 
+  #
+  def data_deletions(self, options):
+    url = "%s://%s/api/data_deletions/sessions" %(self.service_protocol, self.service_url)
+
+    # API accepts UTC Datetime Object so we must convert it to 
+    # UTC seconds as that is what is required by the proctorserve api
+
+    if options.has_key('lower_bound'):
+      options['lower_bound'] = self.convert_times_to_integer(options['lower_bound']) 
+    
+    if options.has_key('upper_bound'):
+      options['upper_bound'] = self.convert_times_to_integer(options['upper_bound']) 
+
+    rest = RestRequestClient()
+    response = rest.make_post_request(url, self.api_identifier, self.shared_secret, options)
+
+    return self.data_deletions_response(response)
+
+
   # Makes a reservation for a session if the time passed in options is a schedulable time.
   #
   # This method will raise exceptions related to authentication (InvalidClientException, InvalidSignatureException, InvalidIpException) or not providing required data (MissingRequiredParameterException)
@@ -266,18 +385,61 @@ class ProctorservApi(object):
     else:
       self.raise_exception_if_necessary(parsed_response)
 
+  def data_retrievals_response(self, response):
+    """ parses through an http response, returns
+    requested session information or raises an exception """
+    parsed_response = response.json()
+    
+    if response.status_code == 200: 
+      return parsed_response
+    else:
+      self.raise_exception_if_necessary(parsed_response)
+
+  def data_deletions_response(self, response):
+    """ parses through an http response, returns
+    session ids associated with deleted session data or raises an exception """
+    parsed_response = response.json()
+
+    if response.status_code == 201:
+      return parsed_response
+    else:
+      self.raise_exception_if_necessary(parsed_response)
+
+  def get_access_code_responses(self, response):
+    """ parses through an http response, returns a 
+    access_code, error message, or raises an exception """
+    parsed_response = response.json()
+
+    if response.status_code != 200:  
+      return self.raise_exception_if_necessary(parsed_response)
+    elif "message" in parsed_response:
+      return parsed_response
+      
+    return parsed_response["access_code"]
+
+  def reset_access_code_responses(self, response):
+    """ parses through an http response, returns a new
+    access_code, error message, or raises an exception """
+    parsed_response = response.json()
+
+    if response.status_code != 200:  
+      return self.raise_exception_if_necessary(parsed_response)
+    elif "message" in parsed_response:
+      return parsed_response
+     
+    return parsed_response["access_code"]
+
 
   def cancellation_reponse(self, response):
     """ parses through an http response, returns confirmation
         of session cancellation or raises an exception """
-    parsed_response = response
-
     if response.status_code == 204:  
       return True
     elif response.status_code != 200:
+      parsed_response = response.json()
       self.raise_exception_if_necessary(parsed_response)
-    else:
-      return False
+    
+    return False
 
 
   def session_event_response(self, response):
@@ -312,9 +474,4 @@ class ProctorservApi(object):
     if options.get("duration") > 180:
       return 180
     if options.get("duration") <= 180:
-      return options["duration"]
-
-
-
-
-
+      return options["duration"]
\ No newline at end of file
